const path = require("path"); 
const fs = require("fs"); 
const xmldom = require('xmldom');

const parser = new xmldom.DOMParser();
const serializer = new xmldom.XMLSerializer();

var inputDir = '';
var outputDir = '';

const languages = ['', '_en_us', '_es_es', '_es_cl', '_es_co'];

function checkArguments() {
  var arguments = process.argv;
  if (arguments.length < 4) {
    console.log('Usage node birt-translate.js <input-dir> <output-dir>');
    process.exit();
  } else {
    inputDir = process.argv[2];
    outputDir = process.argv[3];

    if (!fs.existsSync(inputDir)) {
      console.log(`Input directory (${inputDir}) not found!`);
      process.exit();
    }

    if (!fs.existsSync(outputDir)) {
      try {
        fs.mkdirSync(outputDir);
      } catch(err) {
        console.log(`Error creating output directory (${outputDir})!`);
        throw err;
      }
    }
  }
}

var properties = `# Generated by birt-translate at ${(new Date()).toLocaleString()}\n`;

function readReports() {
  fs.readdir(inputDir, function (err, files) {
    if (err) {
      return console.log('Unable to scan directory: ' + err);
    }

    files.forEach(function (file) {
      console.log(`reading ${file}...`);
      if(!(file.includes('connection.js') || file.includes('translatedReports') || file.includes('translationfiles'))){
        var xml = fs.readFileSync(path.join(inputDir, file), 'utf8');
        var root = parser.parseFromString(xml);
        checkLabels(root);
        checkTranslateFunction(root);
        updateTranslateFiles(root);
  
        var xmlOut = serializer.serializeToString(root);
        try {
          fs.writeFileSync(path.join(outputDir,file), xmlOut, 'utf8');
        } catch(err) {
          console.log(`Error writing output file (${file})`);
          throw err;
        }
      }
    });

    try {
      fs.writeFileSync(path.join(outputDir, 'translation.properties'), properties, 'utf8');
    } catch (err) {
      console.log('Error writing properties file');
      throw err;
    }
  });
}

function unicodeEscape(s) {
  return s.replace(/[^0-9|^a-z]/gi, (char) => { 
    return `\\u${(`000${char.charCodeAt(0).toString(16)}`).substr(-4)}`;
  });
}

function checkLabels(root) {
  var labels = root.getElementsByTagName('label');
  labels = Array.prototype.slice.call(labels);

  labels.forEach((label) => {
    var textNode = label.getElementsByTagName('text-property')[0];
    if(isset(()=> textNode.childNodes[0].nodeValue)){
      var nodeValue = textNode.childNodes[0].nodeValue;
      var escapedValue = unicodeEscape(nodeValue);
      textNode.setAttribute('key', nodeValue);
      properties += `${escapedValue}=${escapedValue}\n`;
    }
  });
}

function checkTranslateFunction(root) {
  var regexScapedSpace = /u0020/gm;
  var expressions = root.getElementsByTagName('expression');
  expressions = Array.prototype.slice.call(expressions);

  expressions.forEach((expression) => {
    var expValue = expression.childNodes[0].nodeValue;
    do {
      var translateIdx = expValue.toLowerCase().indexOf('translate(');
      if (translateIdx === -1) break;

      expValue = expValue.substr(translateIdx + ('translate(').length + 1);
      var parentesisIdx = expValue.indexOf(')');
      var text = expValue.substr(0, parentesisIdx-1);
      var escapedValue = unicodeEscape(text).replace(regexScapedSpace, ' ');
      properties += `${escapedValue}=${escapedValue}\n`;
    } while (true);
  });
}

function updateTranslateFiles(root) {

  var reportNode = root.getElementsByTagName('report')[0];
  var simplePropertyNodes = reportNode.getElementsByTagName('simple-property-list');
  simplePropertyNodes = Array.prototype.slice.call(simplePropertyNodes);
  let boolFound = false;
  
  simplePropertyNodes.forEach((node) => {
    if (node.getAttribute('name') === 'includeResource') {
      boolFound = true;
      while (node.childNodes.length > 0) {
        node.removeChild(node.childNodes[0]);
      }
      languages.forEach((lang) => {
        var translationFileNode = root.createElement('value');
        var translationFileTextNode = root.createTextNode(`translationfiles/translation${lang}`);
        translationFileNode.appendChild(translationFileTextNode);
        node.appendChild(translationFileNode);
      });
    }
  });
  
  if(!boolFound){
    var resourceNode = root.createElement('simple-property-list');
    resourceNode.setAttribute('name', 'includeResource');
    languages.forEach((lang) => {
        var translationFileNode = root.createElement('value');
        var translationFileTextNode = root.createTextNode(`translationfiles/translation${lang}`);
        translationFileNode.appendChild(translationFileTextNode);
        resourceNode.appendChild(translationFileNode);
    });
    reportNode.appendChild(resourceNode);
  }

}

function createPopertieFiles() {
  
}

var isset = (accessor) => {
	  try {
	    return typeof accessor() !== 'undefined' && accessor() !== null;
	  } catch (e) {
	    return false;
	  }
	};

checkArguments();
readReports();